@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Team audio-recorder.
+ * Copyright (c) 2011- Osmo Antero.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,6 +17,7 @@
 */
 #include <string.h>
 #include <glib.h>
+#include <glib/gprintf.h>
 #include <gdk/gdk.h>
 #include <gio/gio.h>
 #include <gio/gdesktopappinfo.h>
@@ -39,6 +40,9 @@
 // Send commands to rec-manager.c
 #include "rec-manager-struct.h"
 
+// rec-manager.c
+#include "rec-manager.h"
+
 // List of players
 static GHashTable *g_player_list = NULL;
 
@@ -48,6 +52,9 @@ static void dbus_player_clear_list();
 static void dbus_player_get_saved();
 static void dbus_player_save(MediaPlayerRec *pl);
 static void dbus_player_delete_saved(const gchar *service_name);
+static gboolean dbus_player_msg_is_duplicate(MediaPlayerRec *player);
+
+static const gchar *status_name(gint status) __attribute__((unused));
 
 gboolean add_player_to_list(gchar *desktop_file, gchar *service_name);
 void add_skype();
@@ -83,11 +90,24 @@ static RecorderCommand *convert_data(MediaPlayerRec *pl) {
     TrackInfo *tr = &pl->track;
 
     RecorderCommand *cmd = g_malloc0(sizeof(RecorderCommand));
-    cmd->track = g_strndup(tr->track, MPRIS_STRLEN);
-    cmd->artist = g_strndup(tr->artist, MPRIS_STRLEN);
+
+    cmd->title = g_strndup(tr->title, MPRIS_STRLEN);
+    cmd->artist = g_strndup(tr->artist, MPRIS_STRLEN);  // Actually a list of artists separated by '\v' (should we remove '\v'?)  
     cmd->album = g_strndup(tr->album, MPRIS_STRLEN);
-    cmd->track_len = tr->track_len;
-    cmd->track_pos = tr->track_pos;
+    cmd->genre = g_strndup(tr->genre, MPRIS_STRLEN); // Actually a list of genres separated by '\v' (should we remove '\v'?)
+
+    cmd->albumArtist = g_strndup(tr->albumArtist, MPRIS_STRLEN); // Actually a list of albumArtists separated by '\v' (should we remove '\v'?)
+    cmd->url = g_strndup(tr->url, MPRIS_STRLEN);
+    cmd->artUrl = g_strndup(tr->artUrl, MPRIS_STRLEN);
+
+    cmd->trackId = g_strndup(tr->trackId, MPRIS_STRLEN);
+    cmd->trackNumber = tr->trackNumber;
+
+    cmd->discNumber = tr->discNumber;
+
+    cmd->trackLength = tr->trackLength;
+    cmd->trackPos = tr->trackPos;
+
     cmd->flags = tr->flags;
 
     if (tr->status == PLAYER_STATUS_PAUSED) {
@@ -108,14 +128,154 @@ static RecorderCommand *convert_data(MediaPlayerRec *pl) {
     return cmd;
 }
 
-void dbus_player_process_data(gpointer player) {
+static gboolean dbus_player_msg_is_duplicate(MediaPlayerRec *player) {
+    // Check if player->track (TrackInfo) is duplicate of the previous one.
+    // Return TRUE if duplicate, otherwise FALSE. 
+    if (!player) return FALSE;
+
+    // Track info
+    TrackInfo *tr = &player->track;
+
+    // Track info
+    TrackInfo *prev_tr = &player->prev_track;
+
+    // Check if message is duplicate of previous one
+    //if (tr->trackLength != prev_tr->trackLength) return FALSE;
+    //if (tr->trackPos != prev_tr->trackPos) return FALSE;
+
+    if (tr->trackNumber != prev_tr->trackNumber) {
+        return FALSE;
+    }
+    
+    if (tr->discNumber != prev_tr->discNumber) {
+        return FALSE;
+    }
+
+    if (tr->audioBitrate != prev_tr->audioBitrate) {
+        return FALSE;
+    }
+
+    if (str_compare(tr->title, prev_tr->title, FALSE)) {
+        return FALSE;
+    }
+
+    if (str_compare(tr->artist, prev_tr->artist, FALSE)) {
+        return FALSE;
+    }
+
+    if (str_compare(tr->album, prev_tr->album, FALSE)) {
+        return FALSE;
+    }
+
+    if (str_compare(tr->genre, prev_tr->genre, TRUE)) {
+        return FALSE;
+    }
+
+    if (str_compare(tr->albumArtist, prev_tr->albumArtist, FALSE)) {
+        return FALSE;
+    }
+
+    if (str_compare(tr->url, prev_tr->url, FALSE)) {
+        return FALSE;
+    }
+
+    //if (str_compare(tr->artUrl, prev_tr->artUrl)) return FALSE;
+
+    if (str_compare(tr->trackId, prev_tr->trackId, FALSE)) {
+        return FALSE;
+    }
+
+    if (str_compare(tr->contentCreated, prev_tr->contentCreated, FALSE)) {
+        return FALSE;
+    }
+
+    // This TrackInfo is 100% duplicate of the previous one
+    return TRUE;
+}
+
+void dbus_player_process_data(gpointer player_rec, gboolean restart) {
     // Send message to the rec-manager.c
+
+    MediaPlayerRec *player = (MediaPlayerRec*)player_rec;
     if (!player) return;
 
+    // Current track
+    TrackInfo *tr = &player->track;
+
+    // Previous track (saved track data)
+    TrackInfo *prev_tr = &player->prev_track;
+
+    // Got PLAYING message from media player? 
+    if (tr->status == PLAYER_STATUS_PLAYING) {
+
+        // Note !
+        // Check if this message from DBus is duplicate of previous one.
+        // Some media players send numerous duplicate messages. Messages confuse audio recorder.
+        if (dbus_player_msg_is_duplicate(player)) {
+
+            // Increment duplicate msg count. This is just for debugging!.
+            player->msg_count++;
+
+            // Is recording paused?
+            gboolean is_paused = rec_manager_is_paused();
+
+            if (is_paused) {
+
+                LOG_PLAYER("%s: This DBus message (#%i) is duplicate of previous one. Continuing PAUSED recording.\n", player->app_name, player->msg_count+1);
+           
+                // Continue paused recording
+                rec_manager_continue_recording();
+
+            } else {
+
+                LOG_PLAYER("%s: This DBus message (#%i) is duplicate of previous one, %s. Dropping this message.\n", 
+                        player->app_name, player->msg_count+1, status_name(tr->status));
+            }
+
+            // Is duplicate. Drop this message. 
+            return;
+        }
+
+        
+        // Start new recording
+
+        // Stop the player first?
+        if (restart ) { // || (tr->status == PLAYER_STATUS_STOP)
+            rec_manager_stop_recording();
+        }
+
+        // Save, remember this track for next time !
+        memcpy(prev_tr, tr, sizeof(TrackInfo));
+
+        // Set msg count to 1 (just for debugging)
+        player->msg_count = 1;
+
+        // Continues to LBL_1
+    }
+
+
+    // Got PAUSED message from media player? 
+    if (tr->status == PLAYER_STATUS_PAUSED) {
+        // Do nothing.
+        // Continues to LBL_1
+    }
+
+    // Got STOPPED message from media player? 
+    if (tr->status == PLAYER_STATUS_STOPPED) {
+        // Nullify prev_tr (forget lastly received track data) 
+        memset(prev_tr, '\0', sizeof(TrackInfo));
+    }
+
+// LBL_1:
+
+    // Debug:
+    //dbus_player_debug_print(player);
+
     // Convert MediaPlayerRec to RecorderCommand
     RecorderCommand *cmd = convert_data(player);
 
-    // Send the command. Rec-manager will free the cmd structure after processing.
+    // Send this command to rec-manager queue. 
+    // Rec-manager will free the cmd structure after processing.
     rec_manager_send_command(cmd);
 }
 
@@ -161,8 +321,9 @@ static void dbus_player_disconnect_signals() {
 }
 
 GHashTable *dbus_player_get_list_ref() {
-    if (!g_player_list)
+    if (!g_player_list) {
         g_player_list = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, dbus_player_delete_item);
+    }
 
     return g_player_list;
 }
@@ -207,6 +368,7 @@ void dbus_player_delete_item(gpointer data) {
     g_free(player->exec_cmd);
     g_free(player->app_name);
     g_free(player->icon_name);
+
     g_free(player);
 }
 
@@ -246,6 +408,41 @@ MediaPlayerRec *dbus_player_lookup_service_name(const gchar *service_name) {
     return NULL;
 }
 
+static const gchar *status_name(gint status) {
+static gchar s[80]; 
+
+    s[0] = '\0';
+
+    switch (status) {
+
+    case PLAYER_STATUS_CLOSED:
+        g_sprintf(s, "Status:%d  PLAYER_STATUS_CLOSED (not running)", status);
+        break;
+
+    case PLAYER_STATUS_STOPPED:
+        g_sprintf(s, "Status:%d  PLAYER_STATUS_STOPPED", status);
+        break;
+
+    case PLAYER_STATUS_PAUSED:
+        g_sprintf(s, "Status:%d  PLAYER_STATUS_PAUSED", status);
+        break;
+
+    case PLAYER_STATUS_PLAYING:
+        g_sprintf(s, "Status:%d  PLAYER_STATUS_PLAYING", status);
+        break;
+
+    case PLAYER_STATUS_NOTIFY_MSG:
+        // Simply a msg to the GUI.
+        g_sprintf(s, "Status:%d  PLAYER_STATUS_NOTIFY_MSG", status);
+        break;
+
+    default:
+        g_sprintf(s, "Unknown status:%d\n", status);
+    }
+
+    return &s[0];
+}
+
 void dbus_player_debug_print(MediaPlayerRec *p) {
     if (!p) return;
 
@@ -258,42 +455,30 @@ void dbus_player_debug_print(MediaPlayerRec *p) {
 
     TrackInfo *tr = &p->track;
 
-    switch (tr->status) {
-    case PLAYER_STATUS_CLOSED:
-        LOG_PLAYER("Status:%d  PLAYER_STATUS_CLOSED (not running)\n", tr->status);
-        break;
-
-    case PLAYER_STATUS_STOPPED:
-        LOG_PLAYER("Status:%d  PLAYER_STATUS_STOPPED\n", tr->status);
-        break;
-
-    case PLAYER_STATUS_PAUSED:
-        LOG_PLAYER("Status:%d  PLAYER_STATUS_PAUSED\n", tr->status);
-        break;
-
-    case PLAYER_STATUS_PLAYING:
-        LOG_PLAYER("Status:%d  PLAYER_STATUS_PLAYING\n", tr->status);
-        break;
-
-    case PLAYER_STATUS_NOTIFY_MSG:
-        // Simply a msg to the GUI.
-        LOG_PLAYER("Status:%d  PLAYER_STATUS_NOTIFY_MSG\n", tr->status);
-        break;
-
-    default:
-        LOG_PLAYER("Unknown status:%d\n", tr->status);
-    }
+    LOG_PLAYER("%s\n", status_name(tr->status));
 
     if (tr->status != PLAYER_STATUS_NOTIFY_MSG) {
-        LOG_PLAYER("Track:%s\n", tr->track);
+        LOG_PLAYER("Title:%s\n", tr->title);
         LOG_PLAYER("Artist:%s\n", tr->artist);
         LOG_PLAYER("Album:%s\n", tr->album);
-        LOG_PLAYER("Track length in microsecs:%ld\n", tr->track_len);
-        LOG_PLAYER("Track pos in microsecs:%ld\n", tr->track_pos);
+        LOG_PLAYER("Genre:%s\n", tr->genre);
+        LOG_PLAYER("AlbumArtist:%s\n", tr->albumArtist);
+        LOG_PLAYER("Url:%s\n", tr->url);
+        LOG_PLAYER("ArtUrl:%s\n", tr->artUrl);
+        LOG_PLAYER("TrackId:%s\n", tr->trackId);
+        LOG_PLAYER("ContentCreated:%s\n", tr->contentCreated);
+
+
+        LOG_PLAYER("Track length in microsecs:%ld\n", tr->trackLength);
+        LOG_PLAYER("Track pos in microsecs:%ld\n", tr->trackPos);
         LOG_PLAYER("Flags:%d\n", tr->flags);
+
+        LOG_PLAYER("Msg count:%d\n", p->msg_count);
+
+
     } else {
         // Simply a msg to the GUI.
-        LOG_PLAYER("Message:%s\n", tr->track);
+        LOG_PLAYER("Message:%s\n", tr->title);
     }
 
     LOG_PLAYER("------------------------------\n");
@@ -318,11 +503,8 @@ GHashTable *dbus_player_get_player_list() {
     // Add Rhythmbox manually (check if installed)
     add_player_to_list("rhythmbox", "org.mpris.MediaPlayer2.rhythmbox");
 
-    // Add Banshee manually (check if installed)
-    add_player_to_list("banshee", "org.mpris.MediaPlayer2.banshee");
-
     // Add Skype manually (check if installed)
-    add_skype();
+	add_skype();
 
     return g_player_list;
 }
@@ -339,7 +521,7 @@ gboolean add_player_to_list(gchar *desktop_file, gchar *service_name) {
     // Set desktop file
     player->desktop_file = g_strdup(desktop_file);
 
-    // Read name and exec_cmd from player's .desktop file
+    // Read rest from player's .desktop file
     get_details_from_desktop_file(player, desktop_file);
 
     // Find executable /usr/bin/exec_cmd
@@ -352,16 +534,10 @@ gboolean add_player_to_list(gchar *desktop_file, gchar *service_name) {
     }
     g_free(path);
 
-
-    // Set icon name
-    // TODO: We should use g_app_info_get_icon(GAppInfo *appinfo) instead.
-    // FIX ME.
-    player->icon_name = g_strdup(desktop_file);
-
     // Function to connect/disconnect event signals for this player
     player->func_set_signals = mpris2_set_signals;
 
-    // Function to get track-info (album, track/song name/title, genre, etc.)
+    // Function to get track-info (album, title/song name/title, genre, etc.)
     player->func_get_info = mpris2_get_metadata;
 
     // Function to start/run the media player
@@ -416,7 +592,7 @@ void add_skype() {
     // Function to register/unregister notification methods
     player->func_set_signals = skype_setup;
 
-    // Function to get track-info (strictly, Skype does not have "track" data, though it has target filename)
+    // Function to get track-info (strictly, Skype does not have "title" data, though it has target filename)
     player->func_get_info = skype_get_info;
 
     // Function to start/run Skype
@@ -435,7 +611,9 @@ void add_skype() {
 void dbus_player_send_notification(gchar *msg) {
     RecorderCommand *cmd = g_malloc0(sizeof(RecorderCommand));
     cmd->type = RECORDING_NOTIFY_MSG;
-    cmd->track = g_strndup(msg, MPRIS_STRLEN);
+
+    // Convey message in the title field
+    cmd->title = g_strndup(msg, MPRIS_STRLEN);
 
     // Send command to rec-manager.c.
     // It will free the cmd structure after processing.
@@ -626,10 +804,11 @@ LBL_1: {
         str_list_free(list);
         list = NULL;
     }
-
 }
 
 void get_details_from_desktop_file(MediaPlayerRec *pl, const gchar *desktop_file) {
+    // Find AppInfo for the given .desktop file
+    // I assume here that .desktop filenames are in ascii (not multibyte) strings
 
     if (!desktop_file) {
         goto LBL_1;
@@ -647,14 +826,82 @@ void get_details_from_desktop_file(MediaPlayerRec *pl, const gchar *desktop_file
 
     // Get GDesktopAppInfo from propgram.desktop file
     GDesktopAppInfo *app_info = g_desktop_app_info_new(s);
-    g_free(s);
 
-    if (!app_info) {
-        goto LBL_1;
+    gchar *found_str = NULL;
+
+    // .desktop file was found?
+    if (!G_IS_DESKTOP_APP_INFO(app_info)) {
+        // No. Search for it.     
+
+        g_free(s); s = NULL;
+
+        // Find application's basename 
+        // Eg. "amarok" of "org.kde.amarok"
+        gchar *p = strrchr(desktop_file, '.');
+
+        if (p && (str_length0(p) > 1)) {
+            s = g_strdup(p + 1);
+        } else {
+            s = g_strdup(desktop_file);
+        }
+
+        gchar ***results = g_desktop_app_info_search(s);
+        // results: gchar *results[][]
+        // results is a 2-dim array of gchar* strings, the first (i dimension) contains results (another array) for each search string we passed in (in s).
+        // Please see: https://developer.gnome.org/gio/stable/gio-Desktop-file-based-GAppInfo.html#g-desktop-app-info-search
+        
+        gchar *s2 = g_strdup_printf("%s.desktop", s);
+
+        for (guint i=0; results && results[i] != NULL; i++) { 
+
+            for (guint j=0; results[i][j] != NULL; j++) {
+        
+                // Some nitpicking here
+                gchar *s1_down = g_ascii_strdown(results[i][j], str_length0(results[i][j]));
+                gchar *s2_down = g_ascii_strdown(s2, str_length0(s2));
+
+                if (g_strrstr(s1_down, s2_down)) {
+                    // Probably safe with utf8 (multibyte)
+                    // https://developer.gnome.org/glib/stable/glib-String-Utility-Functions.html#g-strrstr
+                    found_str = g_strdup(results[i][j]);
+                }
+
+                g_free(s1_down);
+                g_free(s2_down);
+
+                if (found_str) {
+                    break;
+                }
+
+            } // j...
+
+        } // i...
+
+        g_free(s2);
+        
+        // Free gchar *results[][]
+        // Be carefull
+        for (guint i=0; results[i] != NULL; i++) {
+            g_strfreev(results[i]);
+        }      
+        g_free(results); 
+        results = NULL;
+
+        // Try again      
+        if (found_str) {
+            // Load AppInfo
+            app_info = g_desktop_app_info_new(found_str);
+        }            
+
     }
 
-    // Ref:https://developer.gnome.org/gio/2.26/GAppInfo.html
-    // and its implementation:https://developer.gnome.org/gio/2.30/gio-Desktop-file-based-GAppInfo.html
+    g_free(found_str);
+    g_free(s);
+
+    // Do we have AppInfo?
+    if (!G_IS_DESKTOP_APP_INFO(app_info)) {
+        goto LBL_1;
+    }
 
     // Read application title
     const gchar *app_name = g_app_info_get_name(G_APP_INFO(app_info));
@@ -667,34 +914,75 @@ void get_details_from_desktop_file(MediaPlayerRec *pl, const gchar *desktop_file
     // Read executable command and its arguments
     pl->exec_cmd = g_strdup((gchar*)g_app_info_get_commandline(G_APP_INFO(app_info)));
 
-    // Set icon name (notice: this is not a perfect solution but works in most cases!).
-    // TODO: We should use g_app_info_get_icon(GAppInfo *appinfo) instead.
-    // FIX ME.
-    pl->icon_name = g_strdup(desktop_file);
+    pl->icon_name = g_desktop_app_info_get_string(app_info, "Icon");
+    if (!pl->icon_name){
+        pl->icon_name = g_strdup((gchar *)g_app_info_get_executable(G_APP_INFO(app_info)));    
+    }
 
     g_object_unref(app_info);
 
 
+
+    // Remove %U from the exec command.
+    // %u, %U in the exec command (.desktop file) is normally replaced by a file argument 
+    // Some media players show an error if the file argument is empty
+    str_trim(pl->exec_cmd);
+
+    gchar *p = g_strrstr(pl->exec_cmd, "%U");
+    if (!p)
+        p = g_strrstr(pl->exec_cmd, "%u");
+
+    if (p)
+        *p = '\0'; 
+
 LBL_1: {
-        // Take "vlc" from "org.mpris.MediaPlayer2.vlc"
+        // Basename 
+        // Eg. take "vlc" from "org.mpris.MediaPlayer2.vlc"
         gchar *base_name = get_base_name(pl->service_name);
 
         // Make sure these values are set
-        if (str_length0(pl->app_name) < 1) {
+        if (!pl->app_name) {
             pl->app_name = g_strdup(base_name);
         }
 
-        if (str_length0(pl->desktop_file) < 1) {
+        if (!pl->desktop_file) {
             pl->desktop_file = g_strdup(base_name);
         }
 
-        if (str_length0(pl->exec_cmd) < 1) {
+        if (!pl->exec_cmd) {
             pl->exec_cmd = g_strdup(base_name);
         }
 
+        if (!pl->exec_cmd) {
+            pl->exec_cmd = g_strdup(base_name);
+        }
+
+        if (!pl->icon_name) {
+            pl->icon_name = g_strdup(base_name);
+        }
+
         g_free(base_name);
 
     }
 }
 
+void dbus_player_reset_values(gchar *audio_source) {
+    // Reset audio_source's internal values (in dbus-player.c module).
+
+    // Get MediaPlayerRec for this audio_source (service name)
+    MediaPlayerRec *player = dbus_player_lookup_service_name(audio_source);
+
+    if (!player) {
+        return;
+    }    
+
+    // Nullify player->prev_track (TrackInfo record).
+    // User has stopped recording from the GUI (Button or Menu), 
+    // We must clear MediaPlayerRec`s prev_track record.
+    // User can later re-start recording from the media-player, and it is not counted as duplicate START_RECORDING message. 
+    // This is important for some media players.
+    TrackInfo *tr = &player->prev_track;
+    memset(tr, '\0', sizeof(TrackInfo));
+}
+
 
