@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Team audio-recorder.
+ * Copyright (c) 2011- Osmo Antero.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -1006,8 +1006,8 @@ void skype_start_recording() {
     RecorderCommand *cmd = g_malloc0(sizeof(RecorderCommand));
     cmd->type = RECORDING_START;
 
-    // Set track name
-    cmd->track = g_strndup(g_skype.filename, DEF_STR_LEN);
+    // Set filename as title
+    cmd->title = g_strndup(g_skype.filename, DEF_STR_LEN);
 
     // Set folder name as artist ;-)
     cmd->artist = g_strndup(skype_folder, DEF_STR_LEN);
@@ -1016,8 +1016,8 @@ void skype_start_recording() {
     cmd->album = g_strndup(partner_name, DEF_STR_LEN);
 
     // We do not set these:
-    // cmd->track_pos = 0
-    // cmd->track_len = 0;
+    // cmd->trackPos = 0
+    // cmd->trackLength = 0;
     // cmd->flags = 0;
 
     // Send command to rec-manager.c.
@@ -1054,7 +1054,7 @@ void skype_stop_recording() {
     cmd->type = RECORDING_STOP;
 
     // Set track name
-    cmd->track = g_strndup(g_skype.filename, DEF_STR_LEN);
+    cmd->title = g_strndup(g_skype.filename, DEF_STR_LEN);
 
     // Got DURATION messages and ok?
     if (g_skype.call_duration < 1  && !finished_ok) {
@@ -1093,7 +1093,7 @@ void skype_pause_recording() {
     cmd->type = RECORDING_PAUSE;
 
     // Set track name
-    cmd->track = g_strndup(g_skype.filename, DEF_STR_LEN);
+    cmd->title = g_strndup(g_skype.filename, DEF_STR_LEN);
 
     // Send command to rec-manager.c.
     // It will free the cmd structure after processing.
@@ -1346,7 +1346,9 @@ gchar *skype_get_GET_value(gchar *command) {
 static void skype_GUI_message(gchar *msg) {
     RecorderCommand *cmd = g_malloc0(sizeof(RecorderCommand));
     cmd->type = RECORDING_NOTIFY_MSG;
-    cmd->track = g_strndup(msg, DEF_STR_LEN);
+
+    // Convey message in the title field
+    cmd->title = g_strndup(msg, DEF_STR_LEN);
 
     // Send command to rec-manager.c.
     // It will free the cmd structure after processing.
