@@ -1,5 +1,6 @@
 /*
- * Copyright (c) Team audio-recorder.
+ * Copyright (c) 2002 Iain Holmes.
+ * Copyright (c) 2011- Osmo Antero.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,17 +38,15 @@
 // GStreamer pipeline
 static GstElement *g_pipeline = NULL;
 
-// Flag to test if EOS (end of stream) message was seen
-static gboolean g_got_EOS_message = FALSE;
-
 static gboolean rec_level_message_cb(GstBus *bus, GstMessage *message, void *user_data);
 
 GstElement *rec_create_pipeline(PipelineParms *parms, GError **error);
 
 void rec_set_state_to_null();
 
-static gchar *rec_create_filename(gchar *track, gchar *artist, gchar *album);
-static gchar *rec_generate_unique_filename();
+static gchar *rec_create_filename(gchar *track_title, gchar *artist, gchar *album);
+
+//static gchar *rec_generate_datetime_filename();
 static gchar *check_audio_folder(gchar *audio_folder);
 static gchar *rec_get_profile_id();
 
@@ -130,13 +129,12 @@ void rec_continue_recording()  {
     // Valid pipeline?
     if (!GST_IS_PIPELINE(g_pipeline)) return;
 
-    // Get recording state
     gint state = -1;
     gint pending = -1;
     rec_get_state(&state, &pending);
 
     // Already playing?
-    if (state == GST_STATE_PLAYING) return;
+    //if (state == GST_STATE_PLAYING) return;
 
     LOG_DEBUG("\n--------- rec_continue_recording() ----------\n");
 
@@ -145,8 +143,34 @@ void rec_continue_recording()  {
 
     // Continue recording
     gst_element_set_state(g_pipeline, GST_STATE_PLAYING);
+
 }
 
+const gchar *rec_get_state_name(gint state) {
+    // Return name of the recording state, state of pipeline.
+
+    switch (state) {
+    case GST_STATE_PAUSED:
+        return "RECORDING PAUSED";
+
+    case GST_STATE_VOID_PENDING:
+        return "PENDING, STATE SHIFT";
+
+    case GST_STATE_PLAYING:
+        return "RECORDING ON";
+
+    case GST_STATE_READY:
+        return "IN READY STATE";
+
+    case GST_STATE_NULL:
+        return "RECORDING OFF, NULL STATE";
+
+    default:
+        return "UNKNOWN STATE";
+    }
+}
+
+
 void rec_update_gui() {
     // Recording state has changed (PLAYING, PAUSED, STOPPED/NULL).
 
@@ -157,7 +181,7 @@ void rec_update_gui() {
 gboolean rec_start_recording() {
     LOG_DEBUG("\n--------- rec_start_recording() ----------\n");
 
-    // Get recording state
+    // Get actual recording state now!
     gint state = -1;
     gint pending = -1;
     rec_get_state(&state, &pending);
@@ -168,7 +192,7 @@ gboolean rec_start_recording() {
         // Some players do not send STOP message when they change a track.
         // Audio-recorder simply continues to record to the same file.
         //
-        LOG_DEBUG("gst-recorder.c, rec_start_recording()  FIX THIS.\n");
+        LOG_DEBUG("gst-recorder.c, rec_start_recording()  FIX THIS??.\n");
         return TRUE;
 
         // Is it paused?
@@ -178,37 +202,65 @@ gboolean rec_start_recording() {
         return TRUE;
     }
 
-    // Get data from Gsettings
+    // Get data from Gsettings (this data is saved by rec-manager.c !) 
 
-    gchar *track_name = NULL;
+    gchar *track_title = NULL;
     gchar *artist_name = NULL;
     gchar *album_name = NULL;
+    gchar *url = NULL;
 
     gchar *profile_id = NULL;
 
-    // Track-name sent from media players, Skype
-    conf_get_string_value("track/track-name", &track_name);
-    str_trim(track_name);
-    purify_filename(track_name, TRUE/*purify_all*/);
+    // Track-title sent from media players, Skype 
+    conf_get_string_value("track/track-title", &track_title);
+    str_trim(track_title);
+
+    //Start recording to a new file. track-title=Streaming Data, artist=http:  www radiocity fi oulu, album=Radio City Oulu
+    // xesam:artist': <['http://www.radiocity.fi/oulu']>
 
     // Artist sent from media players
     conf_get_string_value("track/artist-name", &artist_name);
     str_trim(artist_name);
-    purify_filename(artist_name, TRUE/*purify_all*/);
 
     // Album sent from media players
     conf_get_string_value("track/album-name", &album_name);
     str_trim(album_name);
+
+    // Url for a web or radio stream or local file.
+    // Local files begin with file:///
+    // Web urls might begin with http://,  https://, wwww., or event something else
+    conf_get_string_value("track/url", &url);
+    str_trim(url);
+
+#if 0 
+    g_print("\n");
+    g_print("Got track-title:%s\n", track_title);
+    g_print("Got artist:%s\n", artist_name);
+    g_print("Got album:%s\n", album_name);
+    g_print("Got url:%s\n", url);
+    g_print("\n");
+#endif
+
+
+    // Some media players copy stream url to both title and url
+    // Metdata key "xesam:title" has type:s and value:'http://yleuni-f.akamaihd.net/i/yleliveradiohd_5@113882/index_64_a-p.m3u8?sd=10&rebase=on'
+    // Metdata key "xesam:url" has type:s and value:'http://yleuni-f.akamaihd.net/i/yleliveradiohd_5@113882/index_64_a-p.m3u8?sd=10&rebase=on'
+    if (g_strrstr0(track_title, url)) {
+        g_free(track_title);
+        track_title = g_strdup("");
+    }
+
+    purify_filename(track_title, TRUE/*purify_all*/);
+    purify_filename(artist_name, TRUE/*purify_all*/);
     purify_filename(album_name, TRUE/*purify_all*/);
 
+    LOG_DEBUG("Start recording. Got track-title=%s, artist=%s, album=%s, url=%s\n", track_title, artist_name, album_name, url);
+
     // Get last (saved, recorded) filename with full path
     gchar *last_file_name = NULL;
     conf_get_string_value("track/last-file-name", &last_file_name);
     str_trim(last_file_name);
 
-    LOG_DEBUG("Start recording to a new file. track-name=%s, artist=%s, album=%s\n",
-              track_name ? track_name : "<generated automatically>", artist_name, album_name);
-
     // Stop current activity
     rec_stop_recording(FALSE);
 
@@ -223,7 +275,7 @@ gboolean rec_start_recording() {
     // Variables
     gboolean ret = FALSE;
 
-    // Parms to construct a pipeline
+    // Parms to construct a Gstreamer pipeline
     PipelineParms *parms = g_malloc0(sizeof(PipelineParms));
 
     // Append to file?
@@ -232,17 +284,41 @@ gboolean rec_start_recording() {
     if (parms->append && g_file_test(last_file_name, G_FILE_TEST_IS_REGULAR)) {
         // Record to an existing file
         parms->filename = g_strdup(last_file_name);
+        
+        goto LBL_2;
     }
-    // Did we get a track-name from a Media Player, Skype?
-    else if (str_length(track_name, NAME_MAX) > 0) {
-        // We have track, album and artist names. Create filename from these.
-        parms->filename = rec_create_filename(track_name, artist_name, album_name);
 
-    } else  {
-        // Simply generate a new, unique filename from date+time pattern
-        parms->filename = rec_generate_unique_filename();
+    // Some players send "Unknown" text instead of empty string
+    if (artist_name && g_ascii_strncasecmp(artist_name, "Unknown", 7) == 0) {
+        *artist_name = '\0';
     }
 
+    // url start with file:///? (file:/// means a local file)
+    if ((str_length(url, NAME_MAX) > 4)  && (!str_startwith(url, "file://", TRUE))) {
+        // Examples of other urls are:
+        //  http://
+        //  https://
+        //  www.
+        //  magnatune://
+        //  somafm://
+        
+        // Create unique filename based on track_title + artist
+        parms->filename = rec_create_filename(track_title, artist_name, album_name);
+
+    }
+
+    // Did we generated a filename with success?
+    if (str_length(parms->filename, NAME_MAX) < 3) {
+
+        g_free(parms->filename);
+
+        // Simply generate a new, unique filename from a datetime (eg. YYYY-MM-DD HH:MM) pattern
+        parms->filename = rec_create_filename("", "", "");
+
+    }
+
+LBL_2:
+
     // Purify the final filename, remove illegal characters. Edit in place.
     purify_filename(parms->filename, FALSE);
 
@@ -259,7 +335,7 @@ gboolean rec_start_recording() {
         goto LBL_1;
     }
 
-    // Save the last file name (so we can continue from it later on (if append==TRUE))
+    // Save the last file name (so we can continue from it later on (if append==TRUE in the GUI))
     conf_save_string_value("track/last-file-name", parms->filename);
 
     // Get the saved media profile id (aac, mp3, cdlossless, cdlossy, etc).
@@ -307,7 +383,7 @@ gboolean rec_start_recording() {
         // Missing Gstreamer plugin!
 
         // Display error message in the GUI (set red label)
-        rec_manager_set_error_text(err_msg);
+      	rec_manager_set_error_text(err_msg);
 
         LOG_ERROR(err_msg);
 
@@ -319,7 +395,6 @@ gboolean rec_start_recording() {
         goto LBL_1;
     }
 
-
     // Now build a GStreamer pipeline with parms
     GError *error = NULL;
     g_pipeline = rec_create_pipeline(parms, &error);
@@ -347,18 +422,18 @@ gboolean rec_start_recording() {
     LOG_DEBUG("------------------------\n");
 
 LBL_1:
-
     g_free(profile_id);
 
-    g_free(track_name);
+    g_free(track_title);
     g_free(artist_name);
     g_free(album_name);
-
-    g_free(last_file_name);
+    g_free(url);
 
     pipeline_free_parms(parms);
     parms = NULL;
 
+    g_free(last_file_name);
+
     return ret;
 }
 
@@ -398,10 +473,18 @@ void rec_stop_recording(gboolean delete_file) {
 
     LOG_DEBUG("rec_stop_recording(%s)\n", (delete_file ? "delete_file=TRUE" : "delete_file=FALSE"));
 
-    // Send EOS message. This will terminate the stream/file properly. This is very important for ACC (.m4a) files.
-    gst_element_send_event(g_pipeline, gst_event_new_eos());
-    gst_element_send_event(g_pipeline, gst_event_new_eos());
+    // Notes:
+    // The gst_element_send_event() with EOS has been commented out earlier because this app started freeze.
 
+    // 25.Jan.2021 Osmo
+    // I have enable the line again because .m4a files are useless without it. 
+
+    // Send EOS message. 
+    // This will terminate the stream/file properly. This is very important for ACC (.m4a) files.
+    // Same as:
+    // $ gst-launch-1.0 -e
+    //   
+    gst_element_send_event(g_pipeline, gst_event_new_eos());
     g_usleep(GST_USECOND * 5);
 
     // Set pipeline state to NULL
@@ -452,11 +535,11 @@ void rec_stop_and_reset() {
 void rec_get_state(gint *state, gint *pending) {
     /*
     state:
-    GST_STATE_VOID_PENDING
-    GST_STATE_NULL
-    GST_STATE_READY
-    GST_STATE_PAUSED
-    GST_STATE_PLAYING
+     GST_STATE_VOID_PENDING
+     GST_STATE_NULL
+     GST_STATE_READY
+     GST_STATE_PAUSED
+     GST_STATE_PLAYING
     */
     *state = GST_STATE_NULL;
     *pending = GST_STATE_NULL;
@@ -486,15 +569,31 @@ gboolean rec_is_recording() {
     return ret;
 }
 
-static gboolean rec_level_message_cb(GstBus *bus, GstMessage *message, void *user_data) {
-    if (!GST_IS_MESSAGE(message)) return TRUE;
+gboolean rec_is_paused() {
+    // Get recording state
+    gint state = -1;
+    gint pending = -1;
+    rec_get_state(&state, &pending);
+
+    gboolean ret = FALSE;
+
+    // Is paused?
+    switch (state) {
+    case GST_STATE_PAUSED:
+        ret = TRUE;
+        break;
+    }
+    return ret;
+}
+
+static gboolean rec_level_message_cb(GstBus *bus, GstMessage *msg, void *user_data) {
 
     static guint64 last_stream_time_t = 0L;
     static guint64 last_stream_time_fz = 0L;
 
     // Calling with NULL arguments?
     // This will reset the static variables, then exit.
-    if (!message) {
+    if (!msg) {
         last_stream_time_t = 0L;
         last_stream_time_fz = 0L;
         return TRUE;
@@ -502,8 +601,8 @@ static gboolean rec_level_message_cb(GstBus *bus, GstMessage *message, void *use
 
     guint64 stream_time = 0L;
 
-    if (message->type == GST_MESSAGE_ELEMENT) {
-        const GstStructure *s = gst_message_get_structure (message);
+    if (msg->type == GST_MESSAGE_ELEMENT) {
+        const GstStructure *s = gst_message_get_structure(msg);
         const gchar *name = gst_structure_get_name(s);
 
 #if 0 
@@ -756,10 +855,7 @@ static void rec_pipeline_error_cb(GstBus *bus, GstMessage *msg, void *userdata)
 }
 
 static void rec_eos_msg_cb(GstBus *bus, GstMessage *msg, void *userdata) {
-    LOG_DEBUG("Got EOS message. Finishing recording.\n");
-
-    // We've seen EOS. Set g_got_EOS_message to TRUE.
-    g_got_EOS_message = TRUE;
+    LOG_DEBUG("Got End Of Stream (EOS) message. Recording ended.\n");
 }
 
 GstElement *rec_create_pipeline(PipelineParms *parms, GError **error) {
@@ -820,7 +916,7 @@ GstElement *rec_create_pipeline(PipelineParms *parms, GError **error) {
     // Catch error messages
     g_signal_connect(bus, "message::error", G_CALLBACK(rec_pipeline_error_cb), NULL);
 
-    // EOS
+    // End of stream
     g_signal_connect(bus, "message::eos", G_CALLBACK(rec_eos_msg_cb), NULL);
 
     gst_object_unref(bus);
@@ -914,8 +1010,9 @@ static gchar *rec_get_profile_id() {
 }
 
 static gchar *check_audio_folder(gchar *audio_folder) {
-    // Create the folder, this may fail.
-    g_mkdir(audio_folder, 0755);
+    // Create the folder with all sub-paths, this may fail.
+    // 0755 = "rwxr-xr-x";
+    g_mkdir_with_parents(audio_folder, 0755);
 
     // Directory exists?
     if (!g_file_test(audio_folder, G_FILE_TEST_IS_DIR)) {
@@ -929,89 +1026,38 @@ static gchar *check_audio_folder(gchar *audio_folder) {
     return audio_folder;
 }
 
-static gchar *rec_generate_unique_filename() {
-    // Generate a new unique filename from date-time pattern.
+static gchar *rec_create_filename(gchar *track_title, gchar *artist, gchar *album) {
+    // Create filename based on track title and artist (and maybe album)
 
-    // Get saved media profile id
-    gchar *profile_id = rec_get_profile_id();
+    // Typical filenames:
+    // 2019-01-20-09:10:58.mp3                : if track_title == NULL   
+    // Love-you-sooo.mp3                      : if track_title != NULL  (try this first)      
+    // Love-you-sooo-2019-01-20-09:10:58.mp3  : if track_title != NULL  (try this next, if was duplicate filename )   
 
-    // And take its file extension (.ogg, .mp3, .flac, etc)
-    gchar *file_ext = profiles_get_extension(profile_id);
-
-    // Audio folder
-    gchar *audio_folder = get_audio_folder();
-
-    // Check if it is writable
-    audio_folder = check_audio_folder(audio_folder);
-
-    // Pattern to generate a unique filename
-    gchar *filename_pattern = get_filename_pattern();
-
-    LOG_DEBUG("audio_folder=%s file_ext=%s pattern=%s\n", audio_folder, file_ext, filename_pattern);
-
-    gchar *basename = substitute_time_and_date_pattern(filename_pattern);
-
-    // Invalid pattern?
-    if (str_length(basename, 4000) < 1) {
-        g_free(basename);
-
-        // File name not given, set it to "Some filename"
-        basename= g_strdup(_("Some filename"));
-    }
-
-    // Filename + file extension
-    gchar *file_name = g_strdup_printf("%s.%s", basename, (file_ext ? file_ext : "xxx"));
-
-    // Final filename with path
-    gchar *final_name = g_build_filename(audio_folder, file_name, NULL);
-
-    // Free the values
-    g_free(profile_id);
-    g_free(file_ext);
-    g_free(audio_folder);
-    g_free(filename_pattern);
-    g_free(basename);
-
-    LOG_DEBUG("Generated filename is:%s.\n", final_name);
-
-    // The caller should g_free() this value.
-    return final_name;
-}
-
-static gchar *rec_create_filename(gchar *track, gchar *artist, gchar *album) {
-    // Note: we are not using album name here. Maybe in the next version.
+    // Add artist (or album) to the path
+    // Add also audio_folder to the path
 
     gchar *file_name = NULL;
 
-    // Do we have a track name?
-    if (str_length(track, NAME_MAX) < 2) {
-        // No.
-        // Auto-generate a new file name from the date+time pattern
-        return rec_generate_unique_filename();
-    }
-    // Length of track name is > 64 bytes?
-    // For example youtube video ids are very long.
-    else if (str_length(track, NAME_MAX) > 64) {
-        // Auto-generate a new file name from the date+time pattern
-        return rec_generate_unique_filename();
-    }
-
-    // Get Audio/ folder
-    gchar *audio_folder = get_audio_folder();
-    audio_folder = check_audio_folder(audio_folder);
-
     // Get the saved media profile id (aac, mp3, cdlossless, cdlossy, etc).
     gchar *profile_id = rec_get_profile_id();
 
     // Take the file extension from media-profile_id; .ogg, .mp3, etc.
     gchar *file_ext = profiles_get_extension(profile_id);
 
-    // Make track + file extension
-    gchar *fname = g_strdup_printf("%s.%s", track, file_ext);
+    // Get Audio/ folder
+    gchar *audio_folder = get_audio_folder();
+    audio_folder = check_audio_folder(audio_folder);
+
+   // Pattern to generate a unique filename
+    gchar *filename_pattern = get_filename_pattern();
+
+    gchar *basename = substitute_time_and_date_pattern(filename_pattern);
+
     gchar *path = NULL;
 
     // Do we have artist name?
-    if (str_length(artist, PATH_MAX) > 0) {
+    if (str_length(artist, NAME_MAX) > 3) {
         // Build path: audio_folder / artist
         // Eg. "/home/moma/Audio/Salomon Burke/" or from Skype "/home/moma/Audio/Skype recordings/"
         path = g_build_filename(audio_folder, artist, NULL);
@@ -1025,45 +1071,96 @@ static gchar *rec_create_filename(gchar *track, gchar *artist, gchar *album) {
         }
     }
 
-    if (path) {
-        // audio_folder / artist + track.ext
-        file_name = g_build_filename(path, fname, NULL);
-    } else {
-        // audio_folder + track.ext
-        file_name = g_build_filename(audio_folder, fname, NULL);
-    }
+#define MAX_SLEN1 35
+#define MAX_SLEN2 23
 
-    // TODO: Should we delete existing audio files?
-    // Delete existing file
-    g_remove(file_name);
-
-    // Filename was write-protected?
-    if (g_file_test(file_name, G_FILE_TEST_IS_REGULAR)) {
-        // Yes.
-        // Generate a new, unique name by using the pattern
+    // 2 loops
+    for (guint i = 0; i < 2; i++) { 
 
         g_free(file_name);
+        file_name = NULL;
 
-        // Pattern to generate a unique filename (date + time pattern)
-        gchar *filename_pattern = get_filename_pattern();
+        gchar *fname = NULL;
+        gchar *str_tmp = g_strdup(track_title); 
 
-        // Re-build the file_name
-        gchar *t = g_strdup_printf("%s/%s-%s.%s", (path ? path : audio_folder), track, filename_pattern, file_ext);
+        // --------------------------------------
+        // i == 0  (try track-title + file_ext)
+        // --------------------------------------
+        if (i == 0) {
 
-        // Substitute date+time values
-        file_name = substitute_time_and_date_pattern(t);
+            // Do we have track_title?
+            if (str_length0(str_tmp) < 3) {
+                g_free(str_tmp); 
+                str_tmp = NULL;              
+                continue;
+            }
 
-        g_free(t);
-        g_free(filename_pattern);
+            // Cut track title to MAX_SLEN1 characters
+            if (str_length0(str_tmp) > MAX_SLEN1) {
+                *(str_tmp + MAX_SLEN1) = '\0';
+                str_trim(str_tmp);
+            }
+
+            // Try track_title + file extension
+            fname = g_strdup_printf("%s.%s", str_tmp, (file_ext ? file_ext : "xxx"));
+
+        // ----------------------------------------------------------
+        // i == 1  (try track-title + YYYY-MM-DD HH:MM:SS + file_ext)
+        // ----------------------------------------------------------
+        } else {
+
+            // Cut track title to MAX_SLEN2 characters
+            if (str_length0(str_tmp) > MAX_SLEN2) {
+                *(str_tmp + MAX_SLEN2) = '\0';
+                str_trim(str_tmp);
+            }
+
+            // Do we have track_title?
+            if (str_length0(str_tmp) < 3) {            
+                // Try basename + file extension
+                fname = g_strdup_printf("%s.%s", basename, (file_ext ? file_ext : "xxx"));
+
+            } else {
+                // Try track_title + basename + file extension
+                fname = g_strdup_printf("%s-%s.%s", str_tmp, basename, (file_ext ? file_ext : "xxx"));
+            }
+
+        }
+
+        // Add path
+
+        if (path) {
+            // audio_folder / artist + track_title.ext
+            file_name = g_build_filename(path, fname, NULL);
+        } else {
+            // audio_folder + track_title.ext
+            file_name = g_build_filename(audio_folder, fname, NULL);
+        }
+
+        g_free(fname);
+        fname = NULL;
+
+        g_free(str_tmp);
+        str_tmp = NULL;
+
+        // file_name is available?
+        gboolean done = (!g_file_test(file_name, G_FILE_TEST_IS_REGULAR));
+
+        if (done) break;  // exit loop?
     }
 
-    g_free(fname);
+
+    LOG_DEBUG("Generated filename=%s\n", file_name);
+
     g_free(path);
 
     g_free(profile_id);
     g_free(file_ext);
     g_free(audio_folder);
 
+    g_free(filename_pattern);
+    g_free(basename);
+
     // The caller should g_free() this value
     return file_name;
 }
