@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Team audio-recorder.
+ * Copyright (c) 2011- Osmo Antero.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -219,7 +219,7 @@ static void parser_fix_list();
 
 static void parser_print_error(gchar *msg);
 
-static gchar *parser_type_to_str(gint type);
+static gchar *parser_type_to_str(gint type) __attribute__((unused));
 
 void parser_module_init() {
     LOG_DEBUG("Init timer-parser.c.\n");
