@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Team audio-recorder.
+ * Copyright (c) 2011- Osmo Antero.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -77,7 +77,11 @@ gboolean is_file_writable(gchar *filename) {
     // Test if filename is writable
     GFileOutputStream *fstream;
     GError *error = NULL;
+
+    if (!filename) return FALSE;
+
     GFile *file = g_file_new_for_path(filename);
+
     gboolean del = FALSE;
 
     if (g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
@@ -438,6 +442,10 @@ gchar *find_command_path(gchar *command) {
     // An example:
     // find_command_path("gimp") will return "/usr/bin/gimp".
 
+    if (!command) {
+        return NULL;
+    }
+
     gchar *line = g_strdup_printf("which %s", command);
 
     gint status;
@@ -780,7 +788,6 @@ gchar *get_audio_folder() {
 
         g_free(home);
         g_free(path);
-        // g_mkdir(folder_name, 0755/* rwxr-xr-x */);
     }
 
     // The caller should g_free() this value
@@ -930,6 +937,8 @@ GdkPixbuf *load_icon_pixbuf(gchar *icon_name, guint _size) {
         return NULL;
     }
 
+    // Normally icon name should equal exec name (without path) in .desktip files 
+
     // Current icon theme
     GtkIconTheme *theme = gtk_icon_theme_get_default();
 
@@ -937,46 +946,14 @@ GdkPixbuf *load_icon_pixbuf(gchar *icon_name, guint _size) {
     pixbuf = gtk_icon_theme_load_icon(theme, icon_name, _size, 0, NULL);
 
     // Got it?
-    if (GDK_IS_PIXBUF(pixbuf)) {
-        goto LBL_1;
+    if (!GDK_IS_PIXBUF(pixbuf)) {
+		// Some media players set entire path for the icon in .desktop file (this is probably a mistake)
+        // Like: /snap/imsplayer/34/usr/share/imsplayer/icons/imsplayer-128.png
+
+        // Load directly from a file
+        pixbuf = get_pixbuf_from_file(icon_name, _size, _size);
     }
 
-    // Executable name != icon_name.
-
-    // Try to get icon_name from xxxx.desktop file.
-
-    gchar *desktop_file = g_strdup_printf("%s.desktop", icon_name);
-    // Eg. "banshee.desktop"
-
-    // Find its full path, eg. /usr/share/applications/banshee.desktop
-    const gchar *filename = NULL;
-    GDesktopAppInfo *app_info = g_desktop_app_info_new(desktop_file);
-    if (app_info) {
-        filename = g_desktop_app_info_get_filename(app_info);
-    }
-
-    // Example: /usr/share/applications/nettfy.desktop:
-    // [Desktop Entry]
-    // Name=Nettfy player
-    // GenericName=Music Player
-    // Comment=Listen to music using nettfy
-    // Icon=nettfy-linux-64x64
-    // Exec=nettfy-bin %U
-
-    // Get real icon name from desktop file (Icon=)
-    gchar *icon_n = read_value_from_keyfile((gchar*)filename, "Desktop Entry", "Icon");
-
-    if (icon_n) {
-        // Load icon
-        pixbuf = gtk_icon_theme_load_icon(theme, icon_n, _size, 0, NULL);
-    }
-
-    g_free(icon_n);
-    g_free(desktop_file);
-    g_object_unref(app_info);
-
-LBL_1:
-
     // Some icons are large. Force to _size.
     if (GDK_IS_PIXBUF(pixbuf)) {
         GdkPixbuf *img = gdk_pixbuf_scale_simple(pixbuf, _size, _size, GDK_INTERP_HYPER);
@@ -1037,6 +1014,74 @@ void kill_frozen_instances(gchar *program_path, GPid preserve_pid) {
     g_free(app_base);
 }
 
+gboolean str_startwith(const gchar *s, const gchar *what, gboolean case_insensitive) {
+    // Check if utf8 string 's' starts with 'what'  
+    size_t len1 = g_utf8_strlen(s, MAX_PATH_LEN);
+    size_t len2 = g_utf8_strlen(what, MAX_PATH_LEN);
+
+    if (len2 > len1) {
+        return FALSE;    
+    }
+
+    gint ret = 0;
+    gchar *s_tmp = NULL;
+    gchar *what_tmp = NULL;
+
+    if (case_insensitive) {
+        s_tmp = g_utf8_casefold(s, len2);
+        what_tmp = g_utf8_casefold(what, len2);
+    } else {
+        s_tmp = g_strndup(s, len2);
+        what_tmp = g_strndup(what, len2);
+    }
+
+    // gchar *g_utf8_substring (const gchar *str, glong start_pos, glong end_pos);
+
+    ret = g_utf8_collate(s_tmp, what_tmp);
+
+    g_free(s_tmp);
+    g_free(what_tmp);
+
+    return (ret == 0);
+}
+
+gint str_compare(const gchar *s1, const gchar *s2, gboolean case_insensitive) {
+
+    if (s1 == NULL && s2 == NULL) {
+        // Assume equals
+        return 0;
+    } else if (s1 == NULL) {
+        // s1 < s2
+        return -1;
+    } else if (s2 == NULL) {
+        // s1 > s2
+        return 1;
+    }
+
+    // Here both are valid (not NULL) strings
+    if (*s1 == '\0' && *s2 == '\0') {
+        // Equals
+        return 0;   
+    } 
+
+    gint ret = 0;
+
+    if (case_insensitive) { 
+        gchar *ss1 = g_utf8_casefold(s1, g_utf8_strlen(s1, MAX_PATH_LEN - 4));
+        gchar *ss2 = g_utf8_casefold(s2, g_utf8_strlen(s2, MAX_PATH_LEN - 4));
+    
+        ret = g_utf8_collate(ss1, ss2);
+        g_free(ss1);
+        g_free(ss2);
+
+    } else {
+
+        ret = g_utf8_collate(s1, s2);
+    }
+
+    return ret;
+}
+
 /*
 gchar *get_command_and_name(gchar *desktop_file, gchar **command) {
     // Load program.desktop file and return application name and executable command (with args).
